﻿// Copyright 2021 faddenSoft. All Rights Reserved.
// See the LICENSE.txt file for distribution terms (Apache 2.0).

using System;
using System.Collections.Generic;
using PluginCommon;

namespace Apple2_ScottAdams {
    /// <summary>
    /// The Scott Adams adventure intepreter uses inline calls to load a 16-bit value into
    /// zero-page memory.  Most of the time it's an address, sometimes it's a constant.
    /// </summary>
    public class InlineCallFormatter : MarshalByRefObject, IPlugin, IPlugin_SymbolList,
            IPlugin_InlineJsr {
        private IApplication mAppRef;
        private byte[] mFileData;

        private const string ADDR_PREFIX = "InA_";
        private enum InlineKind { Unknown = 0, Address };

        private Dictionary<int, InlineKind> mInlineLabels = new Dictionary<int, InlineKind>();

        public string Identifier {
            get {
                return "Scott Adams adventure inline call formatter";
            }
        }

        public void Prepare(IApplication appRef, byte[] fileData, AddressTranslate addrTrans) {
            mAppRef = appRef;
            mFileData = fileData;

            mAppRef.DebugLog("Scott_Formatter(id=" +
                AppDomain.CurrentDomain.Id + "): prepare()");
        }

        public void Unprepare() {
            mAppRef = null;
            mFileData = null;
        }

        public void UpdateSymbolList(List<PlSymbol> plSyms) {
            mInlineLabels.Clear();

            foreach (PlSymbol sym in plSyms) {
                if (sym.Label.StartsWith(ADDR_PREFIX)) {
                    mInlineLabels.Add(sym.Value, InlineKind.Address);
                }
            }
            mAppRef.DebugLog("Prefixes matched " + mInlineLabels.Count + " labels");
        }
        public bool IsLabelSignificant(string beforeLabel, string afterLabel) {
            return DoesPrefixMatch(beforeLabel) || DoesPrefixMatch(afterLabel);
        }

        private static bool DoesPrefixMatch(string label) {
            return label.StartsWith(ADDR_PREFIX);
        }

        public void CheckJsr(int offset, int operand, out bool noContinue) {
            noContinue = false;

            InlineKind kind;
            if (!mInlineLabels.TryGetValue(operand, out kind)) {
                return;
            }

            // Pointer to data.  Format as address.
            if (!Util.IsInBounds(mFileData, offset + 3, 2)) {
                return;
            }
            mAppRef.SetInlineDataFormat(offset + 3, 2,
                DataType.NumericLE, DataSubType.Address, null);
        }
    }
}
